import csv
import json
from http.server import BaseHTTPRequestHandler, HTTPServer
import logging
from datetime import datetime
import re

print()
print("DrayTek Webhook server port 80")
print("Tên file kết quả: report_webhook_port80.csv")
print()
print("Chương trình dùng cho nội bộ để debug - không sử dụng cho các mục đích chính thức")
print()
print()
print()
print("HaLX - 1900.633.641 / hotro@anphat.vn")
print()
print("----------------------www.anphat.vn----------------------")
print()

# Name
csv_filename = "report_webhook_port80.csv"

# control
csv_file_open_for_writing = False

def extract_number_from_string(input_string):
    # get number from string
    match = re.search(r'\d+', input_string)
    if match:
        return int(match.group())
    return 0  # return 0 if number not found  

class S(BaseHTTPRequestHandler):
    def _set_response(self):
        self.send_response(200)
        self.send_header('Content-type', 'text/html')
        self.end_headers()

    def do_GET(self):
        #  Read csv file and send content for get request.
        with open(csv_filename, mode='r') as csv_file:
            csv_data = csv_file.read()

        logging.info("GET request,\nPath: %s\nHeaders:\n%s\n",
                     str(self.path), str(self.headers))
        self._set_response()
        self.wfile.write(csv_data.encode('utf-8'))

    def save_to_csv(self, data):
        global csv_file_open_for_writing

        wan1_tx = extract_number_from_string(data.get("W1_tx(B)", ""))  
        data["W1_tx(B)"] = wan1_tx 
        wan2_tx = extract_number_from_string(data.get("W2_tx(B)", ""))  
        data["W2_tx(B)"] = wan2_tx 

        # wirte
        with open(csv_filename, mode='a', newline='') as csv_file:
            fieldnames = ["timereal", "Device Name", "Uptime", "Memory", "CPU", "W1_status", "W1_IP", "W1_tx(B)", "W1_tx_packet", "W1_tx_rate(bps)", "W1_rx(B)", "W1_rx_packet", "W1_rx_rate(bps)", "W1_sessions", "W2_status", "W2_IP", "W2_tx(B)", "W2_tx_packet", "W2_tx_rate(bps)", "W2_rx(B)", "W2_rx_packet", "W2_rx_rate(bps)", "W2_sessions", "DHCP_total", "DHCP_LAN1", "ARP_total", "ARP_LAN1", "L2LVPN_counts", "Host2LVPN_counts"]

            csv_writer = csv.DictWriter(csv_file, fieldnames=fieldnames, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)

            if not csv_file_open_for_writing:
                csv_writer.writeheader()
                csv_file_open_for_writing = True

            if "timereal" in data:
                csv_writer.writerow(data)
            else:
                print("Warning: 'timereal' is missing in data")

    def do_POST(self):
        # <--- get length core data
        content_length = int(self.headers['Content-Length'])
        # <--- get core data
        post_data = self.rfile.read(content_length)
        logging.info("POST request,\nPath: %s\nHeaders:\n%s\n\nBody:\n%s\n",
                     str(self.path), str(self.headers), post_data.decode('utf-8'))

        try:
            # analysis json
            data = json.loads(post_data.decode('utf-8'))
            device_data = data["device"]
            data["timereal"] = datetime.now().strftime("%Y-%m-%d %H:%M:%S")

            # save to csv
            self.save_to_csv({
                "Device Name": device_data.get("Device Name", ""),
                "Uptime": data["Status"].get("Uptime", ""),
                "Memory": data["Status"].get("Memory", ""),
                "CPU": data["Status"].get("CPU", ""),
                "W1_status": data["Status"].get("WAN1_status", ""),
                "W1_IP": data["Status"].get("WAN1_IP", ""),
                "W1_tx(B)": data["Status"].get("WAN1_tx", ""),
                "W1_tx_packet": data["Status"].get("WAN1_tx_packets", ""),
                "W1_tx_rate(bps)": data["Status"].get("WAN1_tx_rate", ""),
                "W1_rx(B)": data["Status"].get("WAN1_rx", ""),
                "W1_rx_packet": data["Status"].get("WAN1_rx_packets", ""),
                "W1_rx_rate(bps)": data["Status"].get("WAN1_rx_rate", ""),
                "W1_sessions": data["Status"].get("WAN1_sessions", ""),
                "W2_status": data["Status"].get("WAN2_status", ""),
                "W2_IP": data["Status"].get("WAN2_IP", ""),
                "W2_tx(B)": data["Status"].get("WAN2_tx", ""),  # Truy xuất "WAN2_tx" từ dữ liệu JSON
                "W2_tx_packet": data["Status"].get("WAN2_tx_packets", ""),
                "W2_tx_rate(bps)": data["Status"].get("WAN2_tx_rate", ""),
                "W2_rx(B)": data["Status"].get("WAN2_rx", ""),
                "W2_rx_packet": data["Status"].get("WAN2_rx_packets", ""),
                "W2_rx_rate(bps)": data["Status"].get("WAN2_rx_rate", ""),
                "W2_sessions": data["Status"].get("WAN2_sessions", ""),
                "DHCP_total": data["Status"].get("DHCP_total", ""),
                "DHCP_LAN1": data["Status"].get("DHCP_LAN1", ""),
                "ARP_total": data["Status"].get("ARP_total", ""),
                "ARP_LAN1": data["Status"].get("ARP_LAN1", ""),
                "L2LVPN_counts": data["Status"].get("L2LVPN_counts", ""),
                "Host2LVPN_counts": data["Status"].get("Host2LVPN_counts", ""),
                "timereal": data["timereal"]
            })

            self._set_response()
            self.wfile.write("POST request for {}".format(
                self.path).encode('utf-8'))
        except json.JSONDecodeError as e:
            # handle error
            logging.error("Error decoding JSON: %s", str(e))
            self.send_error(400, "Invalid JSON")

def run(server_class=HTTPServer, handler_class=S, addr="0.0.0.0", port=80):
    logging.basicConfig(level=logging.INFO)
    server_address = (addr, port)
    httpd = server_class(server_address, handler_class)
    logging.info('Starting httpd...\n')

    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        pass

    httpd.server_close()
    logging.info('Stopping httpd...\n')

if __name__ == '__main__':
    run()
